/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.operator.keycloak.client;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Optional;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.operator.keycloak.OzgCustomResourceStatus;

class KeycloakClientReconcilerTest {

	public static final String ERROR_MESSAGE = "ErrorMessage";

	@Spy
	@InjectMocks
	private KeycloakClientReconciler reconciler;

	@Mock
	private KeycloakClientService service;

	@Mock
	private KeycloakClientPreconditionService preconditionService;

	@Nested
	class TestReconcile {

		@Test
		void shouldCallServiceAddClient() {
			OzgKeycloakClient client = OzgKeycloakClientTestFactory.create();

			reconciler.reconcile(client, null);

			verify(service).createOrUpdateClient(client.getSpec(), OzgKeycloakClientTestFactory.METADATA_NAMESPACE);
		}

		@Test
		void shouldReturnUpdateStatus() {
			var response = reconciler.reconcile(OzgKeycloakClientTestFactory.create(), null);

			assertThat(response.getResource()).isNotNull();
		}

		@Test
		void shouldSetStatusOk() {
			var response = reconciler.reconcile(OzgKeycloakClientTestFactory.create(), null);

			assertThat(response.getResource().getStatus().getStatus()).isEqualTo(OzgCustomResourceStatus.OK);
		}

		@Test
		void shouldReturnInProgressStatusIfPreconditionsNotMet() {
			when(preconditionService.getReconcilePreconditionErrors(any())).thenReturn(Optional.of(ERROR_MESSAGE));

			var response = reconciler.reconcile(OzgKeycloakClientTestFactory.create(), null);

			assertThat(response.getResource().getStatus().getStatus()).isEqualTo(OzgCustomResourceStatus.IN_PROGRESS);
		}
	}
//
//	@Nested
//	class TestUpdateErrorStatus {
//
//		@Test
//		void shouldSetErrorStatus() {
//			var response = reconciler.updateErrorStatus(OzgKeycloakClientTestFactory.create(), null, new RuntimeException(ERROR_MESSAGE));
//
//			assertThat(response.getResource()).isPresent().map(OzgKeycloakClient::getStatus).map(OzgKeycloakClientStatus::getStatus)
//					.contains(OzgCustomResourceStatus.ERROR);
//		}
//
//		@Test
//		void shouldSetMessage() {
//			var response = reconciler.updateErrorStatus(OzgKeycloakClientTestFactory.create(), null, new RuntimeException(ERROR_MESSAGE));
//
//			assertThat(response.getResource()).isPresent().map(OzgKeycloakClient::getStatus).map(OzgKeycloakClientStatus::getMessage)
//					.contains(ERROR_MESSAGE);
//		}
//	}
}
