package de.ozgcloud.operator.keycloak.user;

import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import io.fabric8.kubernetes.api.model.Secret;
import io.fabric8.kubernetes.api.model.SecretList;
import io.fabric8.kubernetes.client.KubernetesClient;
import io.fabric8.kubernetes.client.dsl.MixedOperation;
import io.fabric8.kubernetes.client.dsl.Resource;

class KubernetesRemoteServiceTest {

	private static final String NAMESPACE = "dummyNamespaceName";

	@InjectMocks
	private KubernetesRemoteService service;
	@Mock
	private KubernetesClient kubernetesClient;

	@DisplayName("Get secret")
	@Nested
	class TestGetSecret {

		private static final String SECRET_NAME = "dummySecretName";

		@Mock
		private MixedOperation<Secret, SecretList, Resource<Secret>> secretsMock;

		@BeforeEach
		void mock() {
			when(kubernetesClient.secrets()).thenReturn(secretsMock);
			when(secretsMock.inNamespace(any())).thenReturn(secretsMock);
		}

		@Test
		void shouldGetFromNamespace() {
			service.getSecret(NAMESPACE, SECRET_NAME);

			verify(secretsMock).inNamespace(NAMESPACE);
		}

		@Test
		void shouldGetWithName() {
			service.getSecret(NAMESPACE, SECRET_NAME);

			verify(secretsMock).withName(SECRET_NAME);
		}
	}
}
