package de.ozgcloud.operator.keycloak.user;

import static org.assertj.core.api.Assertions.*;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;

import de.ozgcloud.operator.keycloak.user.OzgKeycloakUserSpec.KeycloakUserSpecUser;

public class UserSecretBuilderTest {

	private final static String NAME = "dummyName";
	private final static String NAMESPACE = "dummyNamespace";

	private UserSecretBuilder builder = new UserSecretBuilder();

	@DisplayName("Build")
	@Nested
	class TestBuild {

		private final KeycloakUserSpecUser userSpec = KeycloakUserSpecUserTestFactory.create();

		@Test
		void shouldHaveType() {
			var secret = builder.build(NAME, userSpec, NAMESPACE);

			assertThat(secret.getType()).isEqualTo(UserSecretBuilder.SECRET_TYPE);
		}

		@Test
		void shouldHaveUserName() {
			var secret = builder.build(NAME, userSpec, NAMESPACE);

			assertThat(secret.getStringData()).containsEntry(UserSecretBuilder.SECRET_NAME_FIELD, KeycloakUserSpecUserTestFactory.USERNAME);
		}

		@Test
		void shouldHavePassword() {
			var secret = builder.build(NAME, userSpec, NAMESPACE);

			assertThat(secret.getStringData()).containsEntry(UserSecretBuilder.SECRET_PASSWORD_FIELD,
					KeycloakUserSpecUserTestFactory.PASSWORD);
		}

		@DisplayName("metadata")
		@Nested
		class TestMetaData {

			@Test
			void shouldHaveName() {
				var secret = builder.build(NAME, userSpec, NAMESPACE);

				assertThat(secret.getMetadata().getName()).isEqualTo(NAME);
			}

			@Test
			void shouldHaveNamespace() {
				var secret = builder.build(NAME, userSpec, NAMESPACE);

				assertThat(secret.getMetadata().getNamespace()).isEqualTo(NAMESPACE);
			}
		}
	}
}
